# Copyright 2019-2020 The ASReview Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Deprecated, will be removed in version 1.0"""

import warnings


def _moved_warning(obj, new_location, old_location, *args, **kwargs):
    """Helper function for moved classes of functions"""

    def wrapper():
        warnings.warn(
            f"{old_location} has moved to "
            f"{new_location}",
            FutureWarning,
            2)
        return obj(*args, **kwargs)
    return wrapper
