self.__precacheManifest = (self.__precacheManifest || []).concat([
  {
    "revision": "823c06ece38c52e861ecf3fea8337ca4",
    "url": "/index.html"
  },
  {
    "revision": "a18226d2ce2fe1cfb668",
    "url": "/static/css/2.47271137.chunk.css"
  },
  {
    "revision": "b9a9c0951d0d477fbc1d",
    "url": "/static/css/main.554bd1e9.chunk.css"
  },
  {
    "revision": "a18226d2ce2fe1cfb668",
    "url": "/static/js/2.e2f51673.chunk.js"
  },
  {
    "revision": "b0ed0b8bec6a237e99deff2549c8ca45",
    "url": "/static/js/2.e2f51673.chunk.js.LICENSE.txt"
  },
  {
    "revision": "b9a9c0951d0d477fbc1d",
    "url": "/static/js/main.4dbad139.chunk.js"
  },
  {
    "revision": "c33da3d1e3ca0a5fbc49",
    "url": "/static/js/runtime-main.45e47673.js"
  },
  {
    "revision": "32256c13f961465e768b752ce8bc4330",
    "url": "/static/media/1_Welcome.32256c13.svg"
  },
  {
    "revision": "1b5a2da1648b5a3c43b9e17fcf2f9b70",
    "url": "/static/media/2_SetUp.1b5a2da1.svg"
  },
  {
    "revision": "51d773af64f61274691dbe44ac28172b",
    "url": "/static/media/3_StartReviewing.51d773af.svg"
  },
  {
    "revision": "3f48b3f5e297748eb8332a41aef05ee5",
    "url": "/static/media/4_BenefitFromAIAssisted.3f48b3f5.svg"
  },
  {
    "revision": "9e86f7fbdd39122604bf09cfe449ff60",
    "url": "/static/media/5_DontStress.9e86f7fb.svg"
  },
  {
    "revision": "069e79d9d297d9909011185c70a06c12",
    "url": "/static/media/6_DoneItsYourChoice.069e79d9.svg"
  },
  {
    "revision": "79950303110b931d6ac20414e412f3a8",
    "url": "/static/media/7_PublishYourWork.79950303.svg"
  },
  {
    "revision": "89cd7e0cae5cbbcb172a0300808f6ec1",
    "url": "/static/media/Finished.89cd7e0c.svg"
  },
  {
    "revision": "921f0a0356969c6b931cc4ff23763eed",
    "url": "/static/media/InReview.921f0a03.svg"
  },
  {
    "revision": "93ab475360482c95c3ce9b0c1baabc25",
    "url": "/static/media/SetUp.93ab4753.svg"
  },
  {
    "revision": "38e8a1bce1fad97c7df8bfe64057a71e",
    "url": "/static/media/Wordmark_LAB_colour.38e8a1bc.svg"
  },
  {
    "revision": "5cb7edfceb233100075dc9a1e12e8da3",
    "url": "/static/media/roboto-latin-100.5cb7edfc.woff"
  },
  {
    "revision": "7370c3679472e9560965ff48a4399d0b",
    "url": "/static/media/roboto-latin-100.7370c367.woff2"
  },
  {
    "revision": "f8b1df51ba843179fa1cc9b53d58127a",
    "url": "/static/media/roboto-latin-100italic.f8b1df51.woff2"
  },
  {
    "revision": "f9e8e590b4e0f1ff83469bb2a55b8488",
    "url": "/static/media/roboto-latin-100italic.f9e8e590.woff"
  },
  {
    "revision": "b00849e00f4c2331cddd8ffb44a6720b",
    "url": "/static/media/roboto-latin-300.b00849e0.woff"
  },
  {
    "revision": "ef7c6637c68f269a882e73bcb57a7f6a",
    "url": "/static/media/roboto-latin-300.ef7c6637.woff2"
  },
  {
    "revision": "14286f3ba79c6627433572dfa925202e",
    "url": "/static/media/roboto-latin-300italic.14286f3b.woff2"
  },
  {
    "revision": "4df32891a5f2f98a363314f595482e08",
    "url": "/static/media/roboto-latin-300italic.4df32891.woff"
  },
  {
    "revision": "479970ffb74f2117317f9d24d9e317fe",
    "url": "/static/media/roboto-latin-400.479970ff.woff2"
  },
  {
    "revision": "60fa3c0614b8fb2f394fa29944c21540",
    "url": "/static/media/roboto-latin-400.60fa3c06.woff"
  },
  {
    "revision": "51521a2a8da71e50d871ac6fd2187e87",
    "url": "/static/media/roboto-latin-400italic.51521a2a.woff2"
  },
  {
    "revision": "fe65b8335ee19dd944289f9ed3178c78",
    "url": "/static/media/roboto-latin-400italic.fe65b833.woff"
  },
  {
    "revision": "020c97dc8e0463259c2f9df929bb0c69",
    "url": "/static/media/roboto-latin-500.020c97dc.woff2"
  },
  {
    "revision": "87284894879f5b1c229cb49c8ff6decc",
    "url": "/static/media/roboto-latin-500.87284894.woff"
  },
  {
    "revision": "288ad9c6e8b43cf02443a1f499bdf67e",
    "url": "/static/media/roboto-latin-500italic.288ad9c6.woff"
  },
  {
    "revision": "db4a2a231f52e497c0191e8966b0ee58",
    "url": "/static/media/roboto-latin-500italic.db4a2a23.woff2"
  },
  {
    "revision": "2735a3a69b509faf3577afd25bdf552e",
    "url": "/static/media/roboto-latin-700.2735a3a6.woff2"
  },
  {
    "revision": "adcde98f1d584de52060ad7b16373da3",
    "url": "/static/media/roboto-latin-700.adcde98f.woff"
  },
  {
    "revision": "81f57861ed4ac74741f5671e1dff2fd9",
    "url": "/static/media/roboto-latin-700italic.81f57861.woff"
  },
  {
    "revision": "da0e717829e033a69dec97f1e155ae42",
    "url": "/static/media/roboto-latin-700italic.da0e7178.woff2"
  },
  {
    "revision": "9b3766ef4a402ad3fdeef7501a456512",
    "url": "/static/media/roboto-latin-900.9b3766ef.woff2"
  },
  {
    "revision": "bb1e4dc6333675d11ada2e857e7f95d7",
    "url": "/static/media/roboto-latin-900.bb1e4dc6.woff"
  },
  {
    "revision": "28f9151055c950874d2c6803a39b425b",
    "url": "/static/media/roboto-latin-900italic.28f91510.woff"
  },
  {
    "revision": "ebf6d1640ccddb99fb49f73c052c55a8",
    "url": "/static/media/roboto-latin-900italic.ebf6d164.woff2"
  }
]);