.App{text-align:left}.navBar{background-color:#fc0}.btnGrey{background-color:#bcbcbb}.btnBlue{background-color:#00a6e1}.btnGreen{background-color:#5dd41b}.btnRed{background-color:#df462e}.fontSizeSmall{font-size:90%!important}.fontSizeDefault{font-size:100%!important}.fontSizeLarge{font-size:120%!important}.fontSizeLargest{font-size:140%!important}.recharts-tooltip-wrapper .custom-tooltip{margin:0;line-height:24px;border:1px solid #f5f5f5;background-color:hsla(0,0%,100%,.9);padding:10px}.recharts-tooltip-wrapper .custom-tooltip .label{margin:0;color:#666;font-weight:600}.recharts-tooltip-wrapper .custom-tooltip .intro{border-top:1px solid #f5f5f5;margin:0;color:#666;font-weight:600}.recharts-tooltip-wrapper .custom-tooltip .asreview{margin:0;color:#ffcd00;font-weight:400}.recharts-tooltip-wrapper .custom-tooltip .random{margin:0;color:#000;font-weight:400}.cardHighlight{text-align:center}.Card{margin-bottom:28px}.rotate{-webkit-animation:rotation 8s linear infinite;animation:rotation 8s linear infinite}@-webkit-keyframes rotation{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}to{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}@keyframes rotation{0%{-webkit-transform:rotate(0deg);transform:rotate(0deg)}to{-webkit-transform:rotate(359deg);transform:rotate(359deg)}}
/*# sourceMappingURL=main.554bd1e9.chunk.css.map */